/**************************************************************************\
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
*   GdiPlusBrush.h
*
* Abstract:
*
*   Brush related declarations
*
\**************************************************************************/

#ifndef _GDIPLUSBRUSH_H
#define _GDIPLUSBRUSH_H

#include "GdiPlusEnums.h"
#include "GdiPlusTypes.h"
#include "GdiPlusStubs.h"

typedef struct tagGp_TextureBrush
{
	GP_GPTEXTUREBRUSH * nativeBrush;
	GpStatus lastResult;
}GP_TEXTUREBRUSH;

typedef struct tagGp_Texture
{
	GP_GPTEXTURE * nativeBrush;
	GpStatus lastResult;
}GP_TEXTURE;

typedef struct tagGp_Brush
{
	GP_GPBRUSH * nativeBrush; //GpBrush
	GpStatus lastResult;
}GP_BRUSH;

//----------------------------------------------------------------------------
// Brush methods API calls
//----------------------------------------------------------------------------
GpStatus _stdcall GdipCloneBrush(GP_GPBRUSH *brush, GP_GPBRUSH **cloneBrush);
GpStatus _stdcall GdipDeleteBrush(GP_GPBRUSH *brush);
GpStatus _stdcall GdipGetGpBrushType(GP_GPBRUSH *brush, GpBrushType *type);
GpStatus _stdcall GdipGetBrushType(GP_GPBRUSH *brush, GpBrushType *type);

//----------------------------------------------------------------------------
// Hatch Brush methods
//----------------------------------------------------------------------------
GpStatus _stdcall GdipCreateHatchBrush(GpHatchStyle hatchstyle, ARGB forecol, ARGB backcol, GP_GPHATCH **brush);
GpStatus _stdcall GdipGetGpHatchStyle(GP_GPHATCH *brush, GpHatchStyle *hatchstyle);
GpStatus _stdcall GdipGetHatchForegroundColor(GP_GPHATCH *brush, ARGB* forecol);
GpStatus _stdcall GdipGetHatchBackgroundColor(GP_GPHATCH *brush, ARGB* backcol);

//----------------------------------------------------------------------------
// Texture Brush methods
//----------------------------------------------------------------------------
GpStatus _stdcall GdipCreateTexture(GP_GPIMAGE *image, GpWrapMode wrapmode, GP_GPTEXTURE **texture);
GpStatus _stdcall GdipCreateTexture2(GP_GPIMAGE *image, GpWrapMode wrapmode, REAL x, REAL y, REAL width, REAL height, GP_GPTEXTURE **texture);
GpStatus _stdcall GdipCreateTextureIA(GP_GPIMAGE *image, const GP_GPIMAGEATTRIBUTES *imageAttributes, REAL x, REAL y, REAL width, REAL height, GP_GPTEXTURE **texture);
GpStatus _stdcall GdipCreateTexture2I(GP_GPIMAGE *image, GpWrapMode wrapmode, INT x, INT y, INT width, INT height, GP_GPTEXTURE **texture);
GpStatus _stdcall GdipCreateTextureIAI(GP_GPIMAGE *image, const GP_GPIMAGEATTRIBUTES *imageAttributes, INT x, INT y, INT width, INT height, GP_GPTEXTURE **texture);
GpStatus _stdcall GdipGetTextureTransform(GP_GPTEXTURE *brush, GP_GPMATRIX *matrix);
GpStatus _stdcall GdipSetTextureTransform(GP_GPTEXTURE *brush, const GP_GPMATRIX *matrix);
GpStatus _stdcall GdipResetTextureTransform(GP_GPTEXTURE* brush);
GpStatus _stdcall GdipMultiplyTextureTransform(GP_GPTEXTURE* brush, const GP_GPMATRIX *matrix, GpMatrixOrder order);
GpStatus _stdcall GdipTranslateTextureTransform(GP_GPTEXTURE* brush, REAL dx, REAL dy, GpMatrixOrder order);
GpStatus _stdcall GdipScaleTextureTransform(GP_GPTEXTURE* brush, REAL sx, REAL sy, GpMatrixOrder order);
GpStatus _stdcall GdipRotateTextureTransform(GP_GPTEXTURE* brush, REAL angle, GpMatrixOrder order);
GpStatus _stdcall GdipSetTextureGpWrapMode(GP_GPTEXTURE *brush, GpWrapMode wrapmode);
GpStatus _stdcall GdipGetTextureGpWrapMode(GP_GPTEXTURE *brush, GpWrapMode *wrapmode);
GpStatus _stdcall GdipGetTextureImage(GP_GPTEXTURE *brush, GP_GPIMAGE **image);

//----------------------------------------------------------------------------
// Solid Brush methods
//----------------------------------------------------------------------------
GpStatus _stdcall GdipCreateSolidFill(ARGB color, GP_GPSOLIDFILL **brush);
GpStatus _stdcall GdipSetSolidFillColor(GP_GPSOLIDFILL *brush, ARGB color);
GpStatus _stdcall GdipGetSolidFillColor(GP_GPSOLIDFILL *brush, ARGB *color);

//----------------------------------------------------------------------------
// Texture Brush methods
//----------------------------------------------------------------------------

GpStatus _stdcall GdipCreateTexture(GP_GPIMAGE *image, GpWrapMode wrapmode, GP_GPTEXTURE **texture);
GpStatus _stdcall GdipCreateTexture2(GP_GPIMAGE *image, GpWrapMode wrapmode, REAL x, REAL y, REAL width, REAL height, GP_GPTEXTURE **texture);
GpStatus _stdcall GdipCreateTextureIA(GP_GPIMAGE *image, const GP_GPIMAGEATTRIBUTES *imageAttributes, REAL x, REAL y, REAL width, REAL height, GP_GPTEXTURE **texture);
GpStatus _stdcall GdipCreateTexture2I(GP_GPIMAGE *image, GpWrapMode wrapmode, INT x, INT y, INT width, INT height, GP_GPTEXTURE **texture);
GpStatus _stdcall GdipCreateTextureIAI(GP_GPIMAGE *image, const GP_GPIMAGEATTRIBUTES *imageAttributes, INT x, INT y, INT width, INT height, GP_GPTEXTURE **texture);
GpStatus _stdcall GdipGetTextureTransform(GP_GPTEXTURE *brush, GP_GPMATRIX *matrix);
GpStatus _stdcall GdipSetTextureTransform(GP_GPTEXTURE *brush, const GP_GPMATRIX *matrix);
GpStatus _stdcall GdipResetTextureTransform(GP_GPTEXTURE* brush);
GpStatus _stdcall GdipMultiplyTextureTransform(GP_GPTEXTURE* brush, const GP_GPMATRIX *matrix, GpMatrixOrder order);
GpStatus _stdcall GdipTranslateTextureTransform(GP_GPTEXTURE* brush, REAL dx, REAL dy, GpMatrixOrder order);
GpStatus _stdcall GdipScaleTextureTransform(GP_GPTEXTURE* brush, REAL sx, REAL sy, GpMatrixOrder order);
GpStatus _stdcall GdipRotateTextureTransform(GP_GPTEXTURE* brush, REAL angle, GpMatrixOrder order);
GpStatus _stdcall GdipSetTextureWrapMode(GP_GPTEXTURE *brush, GpWrapMode wrapmode);
GpStatus _stdcall GdipGetTextureWrapMode(GP_GPTEXTURE *brush, GpWrapMode *wrapmode);
GpStatus _stdcall GdipGetTextureImage(GP_GPTEXTURE *brush, GP_GPIMAGE **image);


//----------------------------------------------------------------------------
// LineBrush methods
//----------------------------------------------------------------------------

GpStatus _stdcall GdipCreateLineBrush(const GPPOINTF* point1,
                    const GPPOINTF* point2, ARGB color1, ARGB color2,
                    GpWrapMode wrapMode, GP_GPLINEGRADIENT **lineGradient);
GpStatus _stdcall GdipCreateLineBrushI(const GPPOINTI* point1,
                     const GPPOINTI* point2, ARGB color1, ARGB color2,
                     GpWrapMode wrapMode, GP_GPLINEGRADIENT **lineGradient);
GpStatus _stdcall GdipCreateLineBrushFromRect(const GPRECTF* rect,
                     ARGB color1, ARGB color2, GpLinearGradientMode mode,
                     GpWrapMode wrapMode, GP_GPLINEGRADIENT **lineGradient);
GpStatus _stdcall GdipCreateLineBrushFromRectI(const GPRECTI* rect,
                     ARGB color1, ARGB color2, GpLinearGradientMode mode,
                     GpWrapMode wrapMode, GP_GPLINEGRADIENT **lineGradient);
GpStatus _stdcall GdipCreateLineBrushFromRectWithAngle(const GPRECTF* rect,
                     ARGB color1, ARGB color2, REAL angle,
                     BOOL isAngleScalable, GpWrapMode wrapMode,
                     GP_GPLINEGRADIENT **lineGradient);
GpStatus _stdcall GdipCreateLineBrushFromRectWithAngleI(const GPRECTI* rect,
                     ARGB color1, ARGB color2, REAL angle,
                     BOOL isAngleScalable, GpWrapMode wrapMode,
                     GP_GPLINEGRADIENT **lineGradient);


GpStatus _stdcall GdipSetLineColors(GP_GPLINEGRADIENT *brush, ARGB color1, ARGB color2);
GpStatus _stdcall GdipGetLineColors(GP_GPLINEGRADIENT *brush, ARGB* colors);
GpStatus _stdcall GdipGetLineRect(GP_GPLINEGRADIENT *brush, GPRECTF *rect);
GpStatus _stdcall GdipGetLineRectI(GP_GPLINEGRADIENT *brush, GPRECTI *rect);
GpStatus _stdcall GdipSetLineGammaCorrection(GP_GPLINEGRADIENT *brush, BOOL useGammaCorrection);
GpStatus _stdcall GdipGetLineGammaCorrection(GP_GPLINEGRADIENT *brush, BOOL *useGammaCorrection);
GpStatus _stdcall GdipGetLineBlendCount(GP_GPLINEGRADIENT *brush, INT *count);
GpStatus _stdcall GdipGetLineBlend(GP_GPLINEGRADIENT *brush, REAL *blend, REAL* positions,INT count);
GpStatus _stdcall GdipSetLineBlend(GP_GPLINEGRADIENT *brush, const REAL *blend, const REAL* positions, INT count);
GpStatus _stdcall GdipGetLinePresetBlendCount(GP_GPLINEGRADIENT *brush, INT *count);
GpStatus _stdcall GdipGetLinePresetBlend(GP_GPLINEGRADIENT *brush, ARGB *blend, REAL* positions, INT count);
GpStatus _stdcall GdipSetLinePresetBlend(GP_GPLINEGRADIENT *brush, const ARGB *blend,const REAL* positions, INT count);
GpStatus _stdcall GdipSetLineSigmaBlend(GP_GPLINEGRADIENT *brush, REAL focus, REAL scale);
GpStatus _stdcall GdipSetLineLinearBlend(GP_GPLINEGRADIENT *brush, REAL focus, REAL scale);
GpStatus _stdcall GdipSetLineWrapMode(GP_GPLINEGRADIENT *brush, GpWrapMode wrapmode);
GpStatus _stdcall GdipGetLineWrapMode(GP_GPLINEGRADIENT *brush, GpWrapMode *wrapmode);
GpStatus _stdcall GdipGetLineTransform(GP_GPLINEGRADIENT *brush, GP_GPMATRIX *matrix);
GpStatus _stdcall GdipSetLineTransform(GP_GPLINEGRADIENT *brush, const GP_GPMATRIX *matrix);
GpStatus _stdcall GdipResetLineTransform(GP_GPLINEGRADIENT* brush);
GpStatus _stdcall GdipMultiplyLineTransform(GP_GPLINEGRADIENT* brush, const GP_GPMATRIX *matrix, GpMatrixOrder order);
GpStatus _stdcall GdipTranslateLineTransform(GP_GPLINEGRADIENT* brush, REAL dx, REAL dy,GpMatrixOrder order);
GpStatus _stdcall GdipScaleLineTransform(GP_GPLINEGRADIENT* brush, REAL sx, REAL sy, GpMatrixOrder order);
GpStatus _stdcall GdipRotateLineTransform(GP_GPLINEGRADIENT* brush, REAL angle, GpMatrixOrder order);


//----------------------------------------------------------------------------
// Hatch Brush methods
//----------------------------------------------------------------------------
GpStatus _stdcall GdipCreateHatchBrush(GpHatchStyle hatchstyle, ARGB forecol, ARGB backcol, GP_GPHATCH **brush);
GpStatus _stdcall GdipGetHatchStyle(GP_GPHATCH *brush, GpHatchStyle *hatchstyle);
GpStatus _stdcall GdipGetHatchForegroundColor(GP_GPHATCH *brush, ARGB* forecol);
GpStatus _stdcall GdipGetHatchBackgroundColor(GP_GPHATCH *brush, ARGB* backcol);

//----------------------------------------------------------------------------
// GdiPlusLib.lib functions
//----------------------------------------------------------------------------
VOID GpBrush_Delete(GP_BRUSH * this);
GP_BRUSH GpBrush_Brush(GP_GPBRUSH * nativeBrush, GpStatus status);
GP_BRUSH GpBrush_Clone(GP_BRUSH * this);
GpBrushType GpBrush_GetType(GP_BRUSH * this);
GP_BRUSH GpSolidBrush(GP_BRUSH * this, COLOR * color);
GpStatus GpSolidBrush_GetColor(GP_BRUSH * this, COLOR * color);
GpStatus GpSolidBrush_SetColor(GP_BRUSH * this, COLOR * color);
VOID GpTextureBrush(GP_BRUSH * this, GP_IMAGE * image, GpWrapMode wrapMode = eWrapModeTile);
GP_BRUSH GpTextureBrush_RcF(GP_IMAGE * image, GpWrapMode wrapMode, const RECTF * dstRect);
GP_BRUSH GpTextureBrush_IA(GP_IMAGE * image, const RECTF * dstRect, const GP_IMAGEATTRIBUTES * imageAttributes = NULL);
GP_BRUSH GpTextureBrush_IAI(GP_IMAGE * image, const RECTI * dstRect,const GP_IMAGEATTRIBUTES * imageAttributes = NULL);
GP_BRUSH GpTextureBrush_2RcI(GP_IMAGE * image, GpWrapMode wrapMode, const RECTI * dstRect);
GP_BRUSH GpTextureBrush_2(GP_IMAGE * image, GpWrapMode wrapMode, REAL dstX, REAL dstY, REAL dstWidth, REAL dstHeight);
GP_BRUSH GpTextureBrush_2I(GP_IMAGE * image, GpWrapMode wrapMode, INT dstX, INT dstY, INT dstWidth, INT dstHeight);
GpStatus GpTextureBrush_SetTransform(GP_BRUSH * this, const GP_MATRIX * matrix);
GpStatus GpTextureBrush_GetTransform(GP_BRUSH * this, GP_MATRIX * matrix);
GpStatus GpTextureBrush_ResetTransform(GP_BRUSH * this);
GpStatus GpTextureBrush_MultiplyTransform(GP_BRUSH * this, const GP_MATRIX * matrix, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpTextureBrush_TranslateTransform(GP_BRUSH * this, REAL dx, REAL dy, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpTextureBrush_ScaleTransform(GP_BRUSH * this, REAL sx, REAL sy, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpTextureBrush_RotateTransform(GP_BRUSH * this, REAL angle, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpTextureBrush_SetWrapMode(GP_BRUSH * this, GpWrapMode wrapMode);
GpWrapMode GpTextureBrush_GetWrapMode(GP_BRUSH * this);
GP_IMAGE GpTextureBrush_GetImage(GP_BRUSH * this);
GP_BRUSH GpLinearGradientBrush(const POINTF * point1,const POINTF * point2, const COLOR * color1,const COLOR * color2);
GP_BRUSH GpLinearGradientBrush_PtsI(const POINTI * point1,const POINTI * point2, const COLOR * color1, const COLOR * color2);
GP_BRUSH GpLinearGradientBrush_RcF(const RECTF * rect, const COLOR * color1, const COLOR * color2, GpLinearGradientMode mode);
GP_BRUSH GpLinearGradientBrush_RcI(const RECTI * rect, const COLOR * color1, const COLOR * color2, GpLinearGradientMode mode);
GP_BRUSH GpLinearGradientBrush_RcFA(const RECTF * rect, const COLOR * color1, const COLOR * color2, REAL angle, BOOL isAngleScalable = FALSE);
GP_BRUSH GpLinearGradientBrush_RcIA(const RECTI * rect, const COLOR * color1, const COLOR * color2, REAL angle, BOOL isAngleScalable = FALSE);
GpStatus GpLinearGradientBrush_SetLinearColors(GP_BRUSH * brush, const COLOR * color1, const COLOR * color2);
GpStatus GpLinearGradientBrush_GetLinearColors(GP_BRUSH * this, COLOR * colors);
GpStatus GpLinearGradientBrush_GetRectangleRcF(GP_BRUSH * this, RECTF * rect);
GpStatus GpLinearGradientBrush_GetRectangleRcI(GP_BRUSH * this, RECTI * rect);
GpStatus GpLinearGradientBrush_SetGammaCorrection(GP_BRUSH * this, BOOL useGammaCorrection);
BOOL GpLinearGradientBrush_GetGammaCorrection(GP_BRUSH * this);
INT GpLinearGradientBrush_GetBlendCount(GP_BRUSH * this);
GpStatus GpLinearGradientBrush_SetBlend(GP_BRUSH * this, const REAL * blendFactors, const REAL * blendPositions, INT count);
GpStatus GpLinearGradientBrush_GetBlend(GP_BRUSH * this, REAL * blendFactors, REAL * blendPositions, INT count);
INT GpLinearGradientBrush_GetInterpolationColorCount(GP_BRUSH * this);
GpStatus GpLinearGradientBrush_SetInterpolationColors(GP_BRUSH * this, const COLOR * presetColors, const REAL * blendPositions, INT count);
GpStatus GpLinearGradientBrush_GetInterpolationColors(GP_BRUSH * this, COLOR * presetColors, REAL * blendPositions, INT count);
GpStatus GpLinearGradientBrush_SetBlendBellShape(GP_BRUSH * this, REAL focus, REAL scale = 1.0f);
GpStatus GpLinearGradientBrush_SetBlendTriangularShape(GP_BRUSH * this, REAL focus, REAL scale = 1.0f);
GpStatus GpLinearGradientBrush_SetTransform(GP_BRUSH * this, const GP_MATRIX * matrix);
GpStatus GpLinearGradientBrush_GetTransform(GP_BRUSH * this, GP_MATRIX * matrix);
GpStatus GpLinearGradientBrush_ResetTransform(GP_BRUSH * this);
GpStatus GpLinearGradientBrush_MultiplyTransform(GP_BRUSH * this, const GP_MATRIX * matrix, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpLinearGradientBrush_TranslateTransform(GP_BRUSH * this, REAL dx, REAL dy, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpLinearGradientBrush_ScaleTransform(GP_BRUSH * this, REAL sx, REAL sy, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpLinearGradientBrush_RotateTransform(GP_BRUSH * this, REAL angle, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpLinearGradientBrush_SetWrapMode(GP_BRUSH * this, GpWrapMode wrapMode);
GpWrapMode GpLinearGradientBrush_GetWrapMode(GP_BRUSH * this);
GP_BRUSH GpHatchBrush(GpHatchStyle hatchStyle, const COLOR * foreColor, const COLOR * backColor = 0);
GpHatchStyle GpHatchBrush_GetHatchStyle(GP_BRUSH * this);
GpStatus GpHatchBrush_GetForegroundColor(GP_BRUSH * this, COLOR * color);
GpStatus GetBackgroundColor(GP_BRUSH * this, COLOR *color);


#endif
