/**************************************************************************
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
*   GdiPlusCachedBitmap.h
*
* Abstract:
*
*   CachedBitmap is a representation of an accelerated drawing
*   that has restrictions on what operations are allowed in order
*   to accelerate the drawing to the destination.
*
*   CachedBitmap related declarations
*
**************************************************************************/

#ifndef _GDIPLUSCACHEDBITMAP_H
#define _GDIPLUSCACHEDBITMAP_H

/**************************************************************************
*
* Class Name:
*
*   CachedBitmap
*
* Abstract:
*
*   An object to store a bitmap prepared for rendering on a particular
*   Graphics object. The memory storage for the CachedBitmap is opaque
*   to the other Engine code, so the only operations supported are
*   initializing the data (with a bitmap) and using the graphics to
*   draw it on the screen with an integer offset.
*
*   Look for the class definition in GplusHeaders.h
*
* Created:
*
*   04/23/2000 asecchia
*      Created it.
*
**************************************************************************/
#include "GdiPlusEnums.h"
#include "GdiPlusTypes.h"
#include "GdiPlusStubs.h"


typedef struct tagGp_CachedBitmap
{
    GP_GPCACHEDBITMAP *nativeCachedBitmap;
	GpStatus lastResult;
}GP_CACHEDBITMAP;

//----------------------------------------------------------------------------
// Cached Bitmap APIs
//----------------------------------------------------------------------------

GpStatus _stdcall GdipCreateCachedBitmap(GP_GPBITMAP *bitmap, GP_GPGRAPHICS *graphics,
    GP_GPCACHEDBITMAP **cachedBitmap);

GpStatus _stdcall GdipDeleteCachedBitmap(GP_GPCACHEDBITMAP *cachedBitmap);

GpStatus _stdcall GdipDrawCachedBitmap(GP_GPGRAPHICS *graphics,
    GP_GPCACHEDBITMAP *cachedBitmap, INT x, INT y);

UINT _stdcall GdipEmfToWmfBits(HENHMETAFILE hemf, UINT cbData16,
    LPBYTE pData16,INT iMapMode, INT eFlags);

GpStatus _stdcall GdipDrawCachedBitmap(GP_GPGRAPHICS *graphics, GP_GPCACHEDBITMAP *cachedBitmap,INT x,INT y);
GpStatus _stdcall GdipDeleteCachedBitmap(GP_GPCACHEDBITMAP *cachedBitmap);

//----------------------------------------------------------------------------
// GdiPlusLib.lib functions
//----------------------------------------------------------------------------
GP_CACHEDBITMAP GpCachedBitmap(GP_BITMAP *bitmap, GP_GRAPHICS *graphics);
VOID 			GpCachedBitmap_Delete(GP_CACHEDBITMAP * this);
GpStatus 		GpCachedBitmap_GetLastStatus(GP_CACHEDBITMAP * this);


#endif

