/**************************************************************************
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
* GdiPlusColor.h
*
*   Color related declarations
*
**************************************************************************/

#ifndef _GDIPLUSCOLOR_H
#define _GDIPLUSCOLOR_H

#include "GdiPlusEnums.h"
#include "GdiPlusTypes.h"
#include "GdiPlusStubs.h"


//----------------------------------------------------------------------------
// Color mode
//----------------------------------------------------------------------------

typedef enum tagColorMode
{
    eColorModeARGB32 = 0,
    eColorModeARGB64 = 1
}GpColorMode;

// Shift count and bit mask for A, R, G, B components
enum
{
	eAlphaShift  = 24,
	eRedShift    = 16,
	eGreenShift  = 8,
	eBlueShift   = 0
};

enum
{
	eAlphaMask   = 0xff000000,
	eRedMask     = 0x00ff0000,
	eGreenMask   = 0x0000ff00,
	eBlueMask    = 0x000000ff
};

//----------------------------------------------------------------------------
// Color
//----------------------------------------------------------------------------

// Common color constants

typedef enum tagColors
{
	eAliceBlue            = 0xFFF0F8FF,
	eAntiqueWhite         = 0xFFFAEBD7,
	eAqua                 = 0xFF00FFFF,
	eAquamarine           = 0xFF7FFFD4,
	eAzure                = 0xFFF0FFFF,
	eBeige                = 0xFFF5F5DC,
	eBisque               = 0xFFFFE4C4,
	eBlack                = 0xFF000000,
	eBlanchedAlmond       = 0xFFFFEBCD,
	eBlue                 = 0xFF0000FF,
	eBlueViolet           = 0xFF8A2BE2,
	eBrown                = 0xFFA52A2A,
	eBurlyWood            = 0xFFDEB887,
	eCadetBlue            = 0xFF5F9EA0,
	eChartreuse           = 0xFF7FFF00,
	eChocolate            = 0xFFD2691E,
	eCoral                = 0xFFFF7F50,
	eCornflowerBlue       = 0xFF6495ED,
	eCornsilk             = 0xFFFFF8DC,
	eCrimson              = 0xFFDC143C,
	eCyan                 = 0xFF00FFFF,
	eDarkBlue             = 0xFF00008B,
	eDarkCyan             = 0xFF008B8B,
	eDarkGoldenrod        = 0xFFB8860B,
	eDarkGray             = 0xFFA9A9A9,
	eDarkGreen            = 0xFF006400,
	eDarkKhaki            = 0xFFBDB76B,
	eDarkMagenta          = 0xFF8B008B,
	eDarkOliveGreen       = 0xFF556B2F,
	eDarkOrange           = 0xFFFF8C00,
	eDarkOrchid           = 0xFF9932CC,
	eDarkRed              = 0xFF8B0000,
	eDarkSalmon           = 0xFFE9967A,
	eDarkSeaGreen         = 0xFF8FBC8B,
	eDarkSlateBlue        = 0xFF483D8B,
	eDarkSlateGray        = 0xFF2F4F4F,
	eDarkTurquoise        = 0xFF00CED1,
	eDarkViolet           = 0xFF9400D3,
	eDeepPink             = 0xFFFF1493,
	eDeepSkyBlue          = 0xFF00BFFF,
	eDimGray              = 0xFF696969,
	eDodgerBlue           = 0xFF1E90FF,
	eFirebrick            = 0xFFB22222,
	eFloralWhite          = 0xFFFFFAF0,
	eForestGreen          = 0xFF228B22,
	eFuchsia              = 0xFFFF00FF,
	eGainsboro            = 0xFFDCDCDC,
	eGhostWhite           = 0xFFF8F8FF,
	eGold                 = 0xFFFFD700,
	eGoldenrod            = 0xFFDAA520,
	eGray                 = 0xFF808080,
	eGreen                = 0xFF008000,
	eGreenYellow          = 0xFFADFF2F,
	eHoneydew             = 0xFFF0FFF0,
	eHotPink              = 0xFFFF69B4,
	eIndianRed            = 0xFFCD5C5C,
	eIndigo               = 0xFF4B0082,
	eIvory                = 0xFFFFFFF0,
	eKhaki                = 0xFFF0E68C,
	eLavender             = 0xFFE6E6FA,
	eLavenderBlush        = 0xFFFFF0F5,
	eLawnGreen            = 0xFF7CFC00,
	eLemonChiffon         = 0xFFFFFACD,
	eLightBlue            = 0xFFADD8E6,
	eLightCoral           = 0xFFF08080,
	eLightCyan            = 0xFFE0FFFF,
	eLightGoldenrodYellow = 0xFFFAFAD2,
	eLightGray            = 0xFFD3D3D3,
	eLightGreen           = 0xFF90EE90,
	eLightPink            = 0xFFFFB6C1,
	eLightSalmon          = 0xFFFFA07A,
	eLightSeaGreen        = 0xFF20B2AA,
	eLightSkyBlue         = 0xFF87CEFA,
	eLightSlateGray       = 0xFF778899,
	eLightSteelBlue       = 0xFFB0C4DE,
	eLightYellow          = 0xFFFFFFE0,
	eLime                 = 0xFF00FF00,
	eLimeGreen            = 0xFF32CD32,
	eLinen                = 0xFFFAF0E6,
	eMagenta              = 0xFFFF00FF,
	eMaroon               = 0xFF800000,
	eMediumAquamarine     = 0xFF66CDAA,
	eMediumBlue           = 0xFF0000CD,
	eMediumOrchid         = 0xFFBA55D3,
	eMediumPurple         = 0xFF9370DB,
	eMediumSeaGreen       = 0xFF3CB371,
	eMediumSlateBlue      = 0xFF7B68EE,
	eMediumSpringGreen    = 0xFF00FA9A,
	eMediumTurquoise      = 0xFF48D1CC,
	eMediumVioletRed      = 0xFFC71585,
	eMidnightBlue         = 0xFF191970,
	eMintCream            = 0xFFF5FFFA,
	eMistyRose            = 0xFFFFE4E1,
	eMoccasin             = 0xFFFFE4B5,
	eNavajoWhite          = 0xFFFFDEAD,
	eNavy                 = 0xFF000080,
	eOldLace              = 0xFFFDF5E6,
	eOlive                = 0xFF808000,
	eOliveDrab            = 0xFF6B8E23,
	eOrange               = 0xFFFFA500,
	eOrangeRed            = 0xFFFF4500,
	eOrchid               = 0xFFDA70D6,
	ePaleGoldenrod        = 0xFFEEE8AA,
	ePaleGreen            = 0xFF98FB98,
	ePaleTurquoise        = 0xFFAFEEEE,
	ePaleVioletRed        = 0xFFDB7093,
	ePapayaWhip           = 0xFFFFEFD5,
	ePeachPuff            = 0xFFFFDAB9,
	ePeru                 = 0xFFCD853F,
	ePink                 = 0xFFFFC0CB,
	ePlum                 = 0xFFDDA0DD,
	ePowderBlue           = 0xFFB0E0E6,
	ePurple               = 0xFF800080,
	eRed                  = 0xFFFF0000,
	eRosyBrown            = 0xFFBC8F8F,
	eRoyalBlue            = 0xFF4169E1,
	eSaddleBrown          = 0xFF8B4513,
	eSalmon               = 0xFFFA8072,
	eSandyBrown           = 0xFFF4A460,
	eSeaGreen             = 0xFF2E8B57,
	eSeaShell             = 0xFFFFF5EE,
	eSienna               = 0xFFA0522D,
	eSilver               = 0xFFC0C0C0,
	eSkyBlue              = 0xFF87CEEB,
	eSlateBlue            = 0xFF6A5ACD,
	eSlateGray            = 0xFF708090,
	eSnow                 = 0xFFFFFAFA,
	eSpringGreen          = 0xFF00FF7F,
	eSteelBlue            = 0xFF4682B4,
	eTan                  = 0xFFD2B48C,
	eTeal                 = 0xFF008080,
	eThistle              = 0xFFD8BFD8,
	eTomato               = 0xFFFF6347,
	eTransparent          = 0x00FFFFFF,
	eTurquoise            = 0xFF40E0D0,
	eViolet               = 0xFFEE82EE,
	eWheat                = 0xFFF5DEB3,
	eWhite                = 0xFFFFFFFF,
	eWhiteSmoke           = 0xFFF5F5F5,
	eYellow               = 0xFFFFFF00,
	eYellowGreen          = 0xFF9ACD32
}GpColors;

//----------------------------------------------------------------------------
// GdiPlusLib.lib functions
//----------------------------------------------------------------------------
COLOR GpColor(ARGB argb);
COLOR GpColorA(ARGB argb, BYTE alpha);
COLOR GpColor_3(BYTE r, BYTE g, BYTE b);
VOID 	GpColor_3Set(COLOR * color, BYTE r, BYTE g, BYTE b);
COLOR GpColor_4(BYTE a, BYTE r, BYTE g, BYTE b);
VOID 	GpColor_4Set(COLOR * color, BYTE a, BYTE r, BYTE g, BYTE b);
BYTE 	GpColor_GetAlpha(const COLOR * color);
BYTE 	GpColor_GetRed(const COLOR * color);
BYTE 	GpColor_GetGreen(const COLOR * color);
BYTE 	GpColor_GetBlue(const COLOR * color);
ARGB 	GpColor_GetValue(const COLOR * color);
VOID 	GpColor_SetValue(COLOR * color, const ARGB argb);
VOID 	GpColor_SetFromCOLORREF(COLOR * color, const COLORREF rgb);

#endif
