/**************************************************************************\
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
*   GdiPlusImageAttributes.h
*
* Abstract:
*
*   Class for color adjustment object passed to Graphics.DrawImage
*
\**************************************************************************/

#ifndef _GDIPLUSIMAGEATTRIBUTES_H
#define _GDIPLUSIMAGEATTRIBUTES_H

// class GpImageAttributes;

// There are 5 possible sets of color adjustments:
//          ColorAdjustDefault,
//          ColorAdjustBitmap,
//          ColorAdjustBrush,
//          ColorAdjustPen,
//          ColorAdjustText,

// Bitmaps, Brushes, Pens, and Text will all use any color adjustments
// that have been set into the default ImageAttributes until their own
// color adjustments have been set.  So as soon as any "Set" method is
// called for Bitmaps, Brushes, Pens, or Text, then they start from
// scratch with only the color adjustments that have been set for them.
// Calling Reset removes any individual color adjustments for a type
// and makes it revert back to using all the default color adjustments
// (if any).  The SetToIdentity method is a way to force a type to
// have no color adjustments at all, regardless of what previous adjustments
// have been set for the defaults or for that type.

/*
class ImageAttributes : public GplusBase
{
    friend class Graphics;
    friend class TextureBrush;

public:
*/

#include "GdiPlusEnums.h"
#include "GdiPlusTypes.h"
#include "GdiPlusStubs.h"

typedef struct tagGp_ImageAttributes
{
    GP_GPIMAGEATTRIBUTES * nativeImageAttr;
    GpStatus lastResult;
}GP_IMAGEATTRIBUTES;

//----------------------------------------------------------------------------
// ImageAttributes methods API calls
//----------------------------------------------------------------------------
GpStatus _stdcall GdipCreateImageAttributes(GP_GPIMAGEATTRIBUTES **imageattr);
GpStatus _stdcall GdipCloneImageAttributes(const GP_GPIMAGEATTRIBUTES *imageattr, GP_GPIMAGEATTRIBUTES **cloneImageattr);
GpStatus _stdcall GdipDisposeImageAttributes(GP_GPIMAGEATTRIBUTES *imageattr);
GpStatus _stdcall GdipSetImageAttributesToIdentity(GP_GPIMAGEATTRIBUTES *imageattr, GpColorAdjustType type);
GpStatus _stdcall GdipResetImageAttributes(GP_GPIMAGEATTRIBUTES *imageattr, GpColorAdjustType type);
GpStatus _stdcall GdipSetImageAttributesColorMatrix(GP_GPIMAGEATTRIBUTES *imageattr,
                               GpColorAdjustType type, BOOL enableFlag,
                               const COLORMATRIX* colorMatrix,
                               const COLORMATRIX* grayMatrix,
                               GpColorMatrixFlags flags);

GpStatus _stdcall GdipSetImageAttributesThreshold(GP_GPIMAGEATTRIBUTES *imageattr,
                                GpColorAdjustType type, BOOL enableFlag, REAL threshold);

GpStatus _stdcall GdipSetImageAttributesGamma(GP_GPIMAGEATTRIBUTES *imageattr,
                            GpColorAdjustType type, BOOL enableFlag, REAL gamma);

GpStatus _stdcall GdipSetImageAttributesNoOp(GP_GPIMAGEATTRIBUTES *imageattr, GpColorAdjustType type, BOOL enableFlag);
GpStatus _stdcall GdipSetImageAttributesColorKeys(GP_GPIMAGEATTRIBUTES *imageattr,
                                GpColorAdjustType type,  BOOL enableFlag,
                                ARGB colorLow, ARGB colorHigh);

GpStatus _stdcall GdipSetImageAttributesOutputChannel(GP_GPIMAGEATTRIBUTES *imageattr,
                                    GpColorAdjustType type,
                                    BOOL enableFlag,
                                    GpColorChannelFlags channelFlags);

GpStatus _stdcall GdipSetImageAttributesOutputChannelColorProfile(GP_GPIMAGEATTRIBUTES *imageattr,
                                                GpColorAdjustType type,
                                                BOOL enableFlag,
                                                const WCHAR *colorProfileFilename);

GpStatus _stdcall GdipSetImageAttributesRemapTable(GP_GPIMAGEATTRIBUTES *imageattr,
                                 GpColorAdjustType type, BOOL enableFlag,
                                 UINT mapSize, const COLOR_MAP *map);
GpStatus _stdcall GdipSetImageAttributesWrapMode(GP_GPIMAGEATTRIBUTES *imageAttr,
    				GpWrapMode wrap, ARGB argb, BOOL clamp);

GpStatus _stdcall GdipSetImageAttributesICMMode(GP_GPIMAGEATTRIBUTES *imageAttr, BOOL on);
GpStatus _stdcall GdipGetImageAttributesAdjustedPalette(GP_GPIMAGEATTRIBUTES *imageAttr,
    COLORPALETTE * colorPalette, GpColorAdjustType colorAdjustType);

//---------------------------------------------------------
// GdiPlusLib.lib functions
//---------------------------------------------------------
GP_IMAGEATTRIBUTES GpImageAttributes(VOID);
VOID 	GpImageAttributes_Delete(GP_IMAGEATTRIBUTES * this);
GP_IMAGEATTRIBUTES GpImageAttributes_Clone(GP_IMAGEATTRIBUTES * this);
GpStatus GpImageAttributes_SetToIdentity(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_Reset(GP_IMAGEATTRIBUTES * this,  GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_SetColorMatrix( GP_IMAGEATTRIBUTES * this, const COLORMATRIX *colorMatrix, GpColorMatrixFlags mode, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_ClearColorMatrix(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_SetColorMatrices(GP_IMAGEATTRIBUTES * this, const COLORMATRIX *colorMatrix, const COLORMATRIX *grayMatrix, GpColorMatrixFlags mode, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_ClearColorMatrices(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_SetThreshold(GP_IMAGEATTRIBUTES * this, REAL threshold, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_ClearThreshold(GP_IMAGEATTRIBUTES * this,GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_SetGamma(GP_IMAGEATTRIBUTES * this, REAL gamma, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_ClearGamma(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_SetNoOp(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_ClearNoOp(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_SetColorKey(GP_IMAGEATTRIBUTES * this, const COLOR * colorLow, const COLOR * colorHigh, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_ClearColorKey(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_SetOutputChannel(GP_IMAGEATTRIBUTES * this, GpColorChannelFlags channelFlags, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_ClearOutputChannel(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_SetOutputChannelColorProfile(GP_IMAGEATTRIBUTES * this, const WCHAR *colorProfileFilename,  GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_ClearOutputChannelColorProfile(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_SetRemapTable(GP_IMAGEATTRIBUTES * this, UINT mapSize, const COLOR_MAP *map, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_ClearRemapTable(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type = eColorAdjustTypeDefault);
GpStatus GpImageAttributes_ClearRemapTable(GP_IMAGEATTRIBUTES * this, GpColorAdjustType type);
GpStatus GpImageAttributes_SetBrushRemapTable(GP_IMAGEATTRIBUTES * this, UINT mapSize, const COLOR_MAP *map);
GpStatus GpImageAttributes_ClearBrushRemapTable(GP_IMAGEATTRIBUTES * this);
GpStatus GpImageAttributes_SetWrapMode(GP_IMAGEATTRIBUTES * this,GpWrapMode wrap, const COLOR * color, BOOL clamp = FALSE);
GpStatus GpImageAttributes_GetAdjustedPalette(GP_IMAGEATTRIBUTES * this, COLORPALETTE* colorPalette, GpColorAdjustType colorAdjustType);
GpStatus GpImageAttributes_GetLastStatus(GP_IMAGEATTRIBUTES * this);
VOID GpImageAttributes_SetNativeImageAttr(GP_IMAGEATTRIBUTES * this, GP_GPIMAGEATTRIBUTES* nativeImageAttr);
GP_IMAGEATTRIBUTES GpImageAttributes_ImageAttributes(GP_GPIMAGEATTRIBUTES* imageAttr, GpStatus status);
VOID 	GpImageAttributes_SetNativeImageAttr(GP_IMAGEATTRIBUTES * this, GP_GPIMAGEATTRIBUTES* nativeImageAttr);

#endif
