/**************************************************************************\
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
*   GdiplusMatrix.h
*
* Abstract:
*
*   Matrix related declarations
*
\**************************************************************************/

#ifndef _GDIPLUSMATRIX_H
#define _GDIPLUSMATRIX_H

#include "GdiPlusEnums.h"
#include "GdiPlusTypes.h"
#include "GdiPlusStubs.h"

typedef struct tagGp_Matrix
{
    GP_GPMATRIX * nativeMatrix;
    GpStatus lastResult;
}GP_MATRIX;


//----------------------------------------------------------------------------
// Matrix methods API calls
//----------------------------------------------------------------------------
GpStatus _stdcall GdipCreateMatrix(GP_GPMATRIX **matrix);
GpStatus _stdcall GdipCreateMatrix2(REAL m11, REAL m12, REAL m21, REAL m22, REAL dx,
                                      REAL dy, GP_GPMATRIX **matrix);
GpStatus _stdcall GdipCreateMatrix3(const GPRECTF *rect, const GPPOINTF *dstplg,
                                      GP_GPMATRIX **matrix);
GpStatus _stdcall GdipCreateMatrix3I(const GPRECTI *rect, const GPPOINTI *dstplg,
                                       GP_GPMATRIX **matrix);
GpStatus _stdcall GdipCloneMatrix(GP_GPMATRIX *matrix, GP_GPMATRIX **cloneMatrix);
GpStatus _stdcall GdipDeleteMatrix(GP_GPMATRIX *matrix);
GpStatus _stdcall GdipSetMatrixElements(GP_GPMATRIX *matrix, REAL m11, REAL m12, REAL m21, REAL m22,
                      REAL dx, REAL dy);
GpStatus _stdcall GdipMultiplyMatrix(GP_GPMATRIX *matrix, GP_GPMATRIX* matrix2,
                    GpMatrixOrder order);
GpStatus _stdcall GdipTranslateMatrix(GP_GPMATRIX *matrix, REAL offsetX, REAL offsetY,
                    GpMatrixOrder order);
GpStatus _stdcall GdipScaleMatrix(GP_GPMATRIX *matrix, REAL scaleX, REAL scaleY,
                GpMatrixOrder order);
GpStatus _stdcall GdipRotateMatrix(GP_GPMATRIX *matrix, REAL angle, GpMatrixOrder order);
GpStatus _stdcall GdipShearMatrix(GP_GPMATRIX *matrix, REAL shearX, REAL shearY,
                GpMatrixOrder order);
GpStatus _stdcall GdipInvertMatrix(GP_GPMATRIX *matrix);
GpStatus _stdcall GdipTransformMatrixPoints(GP_GPMATRIX *matrix, GPPOINTF *pts, INT count);
GpStatus _stdcall GdipTransformMatrixPointsI(GP_GPMATRIX *matrix, GPPOINTI *pts, INT count);
GpStatus _stdcall GdipVectorTransformMatrixPoints(GP_GPMATRIX *matrix, GPPOINTF *pts, INT count);
GpStatus _stdcall GdipVectorTransformMatrixPointsI(GP_GPMATRIX *matrix, GPPOINTI *pts, INT count);
GpStatus _stdcall GdipGetMatrixElements(const GP_GPMATRIX *matrix, REAL *matrixOut);
GpStatus _stdcall GdipIsMatrixInvertible(const GP_GPMATRIX *matrix, BOOL *result);
GpStatus _stdcall GdipIsMatrixIdentity(const GP_GPMATRIX *matrix, BOOL *result);
GpStatus _stdcall GdipIsMatrixEqual(const GP_GPMATRIX *matrix, const GP_GPMATRIX *matrix2, BOOL *result);


//--------------------------------------------------------
// GdiPlusLib.lib functions
//--------------------------------------------------------
GP_MATRIX GpMatrix(VOID);
GP_MATRIX GpMatrix_2(REAL m11, REAL m12, REAL m21, REAL m22, REAL dx, REAL dy);
GP_MATRIX GpMatrix_3(const RECTF * rect, const POINTF* dstplg);
GP_MATRIX GpMatrix_3I(const RECTI * rect, const POINTI* dstplg);
VOID GpMatrix_Delete(GP_MATRIX * this);
GP_MATRIX GpMatrix_Clone(GP_MATRIX * this);
GpStatus GpMatrix_GetElements(GP_MATRIX * this, REAL *m);
GpStatus GpMatrix_SetElements(GP_MATRIX * this, REAL m11, REAL m12, REAL m21, REAL m22, REAL dx, REAL dy);
REAL GpMatrix_OffsetX(GP_MATRIX * this);
REAL GpMatrix_OffsetY(GP_MATRIX * this);
GpStatus GpMatrix_Reset(GP_MATRIX * this);
GpStatus GpMatrix_Multiply(GP_MATRIX * this, GP_MATRIX *matrix, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpMatrix_Translate(GP_MATRIX * this, REAL offsetX, REAL offsetY,GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpMatrix_Scale(GP_MATRIX * this, REAL scaleX, REAL scaleY, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpMatrix_Rotate(GP_MATRIX * this, REAL angle, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpMatrix_RotateAt(GP_MATRIX * this, REAL angle, const POINTF * center, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpMatrix_Shear(GP_MATRIX * this, REAL shearX, REAL shearY, GpMatrixOrder order = eMatrixOrderPrepend);
GpStatus GpMatrix_Invert(GP_MATRIX * this);
GpStatus GpMatrix_TransformPointsF(GP_MATRIX * this, POINTF* pts, INT count = 1);
GpStatus GpMatrix_TransformPointsI(GP_MATRIX * this, POINTI* pts, INT count = 1);
GpStatus GpMatrix_TransformVectorsF(GP_MATRIX * this, POINTF* pts, INT count = 1);
GpStatus GpMatrix_TransformVectorsI(GP_MATRIX * this, POINTI* pts, INT count = 1);
BOOL GpMatrix_IsInvertible(GP_MATRIX * this);
BOOL GpMatrix_IsIdentity(GP_MATRIX * this);
BOOL GpMatrix_Equals(GP_MATRIX * this, GP_MATRIX *matrix);
GpStatus GpMatrix_GetLastStatus(GP_MATRIX * this);
GP_MATRIX GpMatrix_Matrix(GP_GPMATRIX *nativeMatrix );
static VOID GpMatrix_SetNativeMatrix(GP_MATRIX * this, GP_GPMATRIX *nativeMatrix);

#endif

