/**************************************************************************\
*
* All files in GdiPlusLib have been modified from the Microsoft
* GdiPlus C++ header files.
*
* Module Name:
*
*   GdiPlusMetafile.h
*
* Abstract:
*
*   Metafile related declarations
*
\**************************************************************************/

#ifndef _GDIPLUSMETAFILE_H
#define _GDIPLUSMETAFILE_H

#include <objidl.h>
#include "GdiPlusEnums.h"
#include "GdiPlusStubs.h"
#include "GdiPlusTypes.h"
#include "GdiPlusGraphics.h"

#include <pshpack2.h>   // set structure packing to 2
#include <poppack.h>
#pragma pack(push,2)

typedef struct tagGdip_APMRect16
{
	INT16           Left;
    INT16           Top;
    INT16           Right;
    INT16           Bottom;
}GP_APMRECT16;

#pragma pack(pop)

typedef struct tagGdip_APMFileHeader
{
    UINT32          Key;            // GP_WMF_ALDUSKEY
    INT16           Hmf;            // Metafile HANDLE number (always 0)
    GP_APMRECT16    BoundingBox;    // Coordinates in metafile units
    INT16           Inch;           // Number of metafile units per inch
    UINT32          Reserved;       // Reserved (always 0)
    INT16           Checksum;       // Checksum value for previous 10 WORDs
}GP_APMFILEHEADER;

typedef BOOL (CALLBACK * EnumerateMetafileProc)(GpEmfPlusRecordType, UINT,UINT,const BYTE*, VOID*);

typedef struct tagGp_Metafile
{
    GP_GPMETAFILE * nativeImage;
    GpStatus lastResult;
}GP_METAFILE;

GpStatus _stdcall GdipEnumerateMetafileDestPoint(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE *  metafile, const POINTF * destPoint,
    EnumerateMetafileProc callback, VOID * callbackData,
    const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileDestPointI(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE * metafile, const POINTI * destPoint,
    EnumerateMetafileProc   callback, VOID * callbackData,
    const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileDestRect(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE *  metafile, const RECTF * destRect,
    EnumerateMetafileProc   callback, VOID * callbackData,
    const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileDestRectI(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE *  metafile, const RECTI * destRect,
	EnumerateMetafileProc   callback, VOID * callbackData,
    const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileDestPoints(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE *  metafile, const POINTF * destPoints,
    INT count, EnumerateMetafileProc callback, VOID * callbackData,
    const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileDestPointsI(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE *  metafile, const POINTI * destPoints,
    INT count, EnumerateMetafileProc callback, VOID * callbackData,
    const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileSrcRectDestPoint(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE * metafile, const POINTF * destPoint,
    const RECTF * srcRect, GpUnit srcUnit, EnumerateMetafileProc callback,
    VOID * callbackData, const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileSrcRectDestPointI(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE *  metafile, const POINTI * destPoint,
    const RECTI * srcRect, GpUnit srcUnit, EnumerateMetafileProc callback,
    VOID * callbackData, const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileSrcRectDestRect(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE * metafile, const RECTF * destRect,
    const RECTF * srcRect, GpUnit srcUnit,
    EnumerateMetafileProc   callback, VOID * callbackData,
    const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileSrcRectDestRectI(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE * metafile, const RECTI * destRect,
    const RECTI * srcRect, GpUnit srcUnit, EnumerateMetafileProc callback,
    VOID * callbackData, const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileSrcRectDestPoints(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE * metafile, const POINTF * destPoints,
    INT count, const RECTF * srcRect, GpUnit srcUnit,
    EnumerateMetafileProc   callback, VOID * callbackData,
    const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipEnumerateMetafileSrcRectDestPointsI(GP_GPGRAPHICS * graphics,
    const GP_GPMETAFILE *  metafile, const POINTI * destPoints,
    INT count, const RECTI * srcRect, GpUnit srcUnit,
    EnumerateMetafileProc   callback, VOID * callbackData,
    const GP_GPIMAGEATTRIBUTES * imageAttributes);

GpStatus _stdcall GdipPlayMetafileRecord(const GP_GPMETAFILE *  metafile,
    GpEmfPlusRecordType recordType, UINT flags, UINT dataSize,
    const BYTE * data);

GpStatus _stdcall GdipCreateMetafileFromWmf(HMETAFILE hWmf, BOOL deleteWmf, const GP_APMFILEHEADER * apmFileHeader, GP_GPMETAFILE **metafile);
GpStatus _stdcall GdipCreateMetafileFromEmf(HENHMETAFILE hEmf, BOOL deleteEmf, GP_GPMETAFILE **metafile);
GpStatus _stdcall GdipCreateMetafileFromFile(const WCHAR* file, GP_GPMETAFILE **metafile);
GpStatus _stdcall GdipCreateMetafileFromWmfFile(const WCHAR* file, const GP_APMFILEHEADER * apmFileHeader, GP_GPMETAFILE **metafile);
GpStatus _stdcall GdipCreateMetafileFromStream(IStream * stream, GP_GPMETAFILE **metafile);
GpStatus _stdcall GdipRecordMetafile(HDC referenceHdc, GpEmfType type, const GPRECTF * frameRect, GpMetafileFrameUnit frameUnit, const WCHAR * description, GP_GPMETAFILE **  metafile);
GpStatus _stdcall GdipRecordMetafileI(HDC referenceHdc, GpEmfType type, const GPRECTI *  frameRect, GpMetafileFrameUnit frameUnit, const WCHAR * description, GP_GPMETAFILE ** metafile);

GpStatus _stdcall GdipRecordMetafileFileName(const WCHAR* fileName, HDC referenceHdc,
    GpEmfType type, const GPRECTF * frameRect, GpMetafileFrameUnit frameUnit,
    const WCHAR *   description, GP_GPMETAFILE ** metafile);

GpStatus _stdcall GdipRecordMetafileFileNameI(const WCHAR* fileName, HDC referenceHdc,
    GpEmfType type, const RECTI * frameRect, GpMetafileFrameUnit frameUnit,
    const WCHAR * description, GP_GPMETAFILE ** metafile);

GpStatus _stdcall GdipGetHemfFromMetafile(GP_GPMETAFILE * metafile, HENHMETAFILE * hEmf);
GpStatus _stdcall GdipCreateStreamOnFile(const WCHAR * filename, UINT access, IStream **stream);
GpStatus _stdcall GdipRecordMetafileStream(IStream * stream, HDC referenceHdc, GpEmfType type,
    const GPRECTF * frameRect, GpMetafileFrameUnit frameUnit, const WCHAR * description,
    GP_GPMETAFILE **  metafile);

GpStatus _stdcall GdipGetMetafileHeaderFromMetafile(GP_GPMETAFILE *  metafile, GP_METAFILEHEADER * header);
GpStatus _stdcall GdipRecordMetafileStreamI(IStream * stream, HDC referenceHdc,
    GpEmfType type,const RECTI * rameRect, GpMetafileFrameUnit frameUnit,
    const WCHAR * description, GP_GPMETAFILE ** metafile);

GpStatus _stdcall GdipGetMetafileHeaderFromEmf(HENHMETAFILE hEmf, GP_METAFILEHEADER * header);
GpStatus _stdcall GdipGetMetafileHeaderFromFile(const WCHAR* filename, GP_METAFILEHEADER * header);
GpStatus _stdcall GdipGetMetafileHeaderFromStream(IStream * stream, GP_METAFILEHEADER * header);

GpStatus _stdcall GdipSetMetafileDownLevelRasterizationLimit(GP_METAFILE * metafile,
    UINT metafileRasterizationLimitDpi);

//-------------------------------------------------------
// GdiPlusLib.lib functions
//-------------------------------------------------------
GP_METAFILE GpMetafile(HMETAFILE hWmf, const GP_APMFILEHEADER * apmFileHeader,  BOOL deleteWmf = FALSE);
GP_METAFILE GpMetafile_FileName(const WCHAR* filename);
GP_METAFILE GpMetafile_Name(const WCHAR* filename, const GP_APMFILEHEADER * apmFileHeader);
GP_METAFILE GpMetafile_IStream(IStream* stream);
GP_METAFILE GpMetafile_ToMem(HDC referenceHdc, GpEmfType type, const WCHAR * description = NULL);
GP_METAFILE GpMetafile_ToMemRcF(HDC referenceHdc, const RECTF * frameRect, GpMetafileFrameUnit frameUnit, GpEmfType type, const WCHAR * description = NULL);
GP_METAFILE Gp_Metafile_ToMemRcI(HDC referenceHdc, const RECTI * frameRect, GpMetafileFrameUnit frameUnit, GpEmfType type, const WCHAR * description = NULL);
GP_METAFILE GpMetafile_ToFile(const WCHAR* fileName, HDC referenceHdc, GpEmfType type, const WCHAR * description = NULL);
GP_METAFILE Gp_Metafile_ToFileRcF(const WCHAR* fileName, HDC referenceHdc,  const RECTF * frameRect, GpMetafileFrameUnit frameUnit,  GpEmfType type, const WCHAR * description = NULL);
GP_METAFILE Gp_Metafile_ToFileRcI(const WCHAR* fileName, HDC referenceHdc, const RECTI * frameRect, GpMetafileFrameUnit frameUnit, GpEmfType type, const WCHAR * description = NULL);
GP_METAFILE GpMetafile_ToStream(IStream * stream, HDC referenceHdc, GpEmfType type, const WCHAR * description = NULL);
GP_METAFILE GpMetafile_ToStreamRcF(IStream * stream, HDC referenceHdc, const RECTF * frameRect, GpMetafileFrameUnit frameUnit,  GpEmfType type, const WCHAR * description = NULL);
GP_METAFILE Gp_Metafile_ToStreamRcI(IStream * stream, HDC referenceHdc, const RECTI * frameRect, GpMetafileFrameUnit frameUnit,  GpEmfType type, const WCHAR * description = NULL);
GpStatus GpMetafile_GetMetafileHeader(HENHMETAFILE hEmf, GP_METAFILEHEADER * header);
GpStatus GpMetafile_GetMetafileHeader_File(const WCHAR* filename, GP_METAFILEHEADER * header);
GpStatus GpMetafile_GetMetafileHeaderStream(IStream * stream, GP_METAFILEHEADER * header);
GpStatus GpMetafile_GetMetafileHeaderHead(GP_METAFILE * this, GP_METAFILEHEADER * header);
HENHMETAFILE  GpMetafile_GethEmFHMETAFILE(GP_METAFILE * this);
GpStatus GpMetafile_PlayRecord(GP_METAFILE * this, GpEmfPlusRecordType recordType, UINT flags, UINT dataSize, const BYTE * data);
VOID GpMetafile_SetNativeImage(GP_METAFILE * this, GP_GPMETAFILE* nativeImage);

#endif // !_METAFILE_H
